import { StoreApi, StoreMutatorIdentifier, StateCreator as StateCreator$1 } from 'zustand';
import { StateCreator } from 'zustand/vanilla';

type onSave<TState> = ((pastState: TState, currentState: TState) => void) | undefined;
interface _TemporalState<TState> {
    pastStates: Partial<TState>[];
    futureStates: Partial<TState>[];
    undo: (steps?: number) => void;
    redo: (steps?: number) => void;
    clear: () => void;
    isTracking: boolean;
    pause: () => void;
    resume: () => void;
    setOnSave: (onSave: onSave<TState>) => void;
    _onSave: onSave<TState>;
    _handleSet: (pastState: TState) => void;
}
interface ZundoOptions<TState, PartialTState = TState> {
    partialize?: (state: TState) => PartialTState;
    limit?: number;
    equality?: (currentState: TState, pastState: TState) => boolean;
    onSave?: onSave<TState>;
    handleSet?: (handleSet: StoreApi<TState>['setState']) => StoreApi<TState>['setState'];
    pastStates?: Partial<PartialTState>[];
    futureStates?: Partial<PartialTState>[];
    wrapTemporal?: (storeInitializer: StateCreator<_TemporalState<TState>, [
        StoreMutatorIdentifier,
        unknown
    ][], [
    ]>) => StateCreator<_TemporalState<TState>, [
        StoreMutatorIdentifier,
        unknown
    ][], [
        StoreMutatorIdentifier,
        unknown
    ][]>;
}
type Write<T, U> = Omit<T, keyof U> & U;
type TemporalState<TState> = Omit<_TemporalState<TState>, '_onSave' | '_handleSet'>;

type Zundo = <TState, Mps extends [StoreMutatorIdentifier, unknown][] = [], Mcs extends [StoreMutatorIdentifier, unknown][] = [], UState = TState>(config: StateCreator$1<TState, [...Mps, ['temporal', unknown]], Mcs>, options?: ZundoOptions<TState, UState>) => StateCreator$1<TState, Mps, [
    ['temporal', StoreApi<TemporalState<UState>>],
    ...Mcs
]>;
declare module 'zustand/vanilla' {
    interface StoreMutators<S, A> {
        temporal: Write<S, {
            temporal: A;
        }>;
    }
}
declare const temporal: Zundo;

export { TemporalState, Zundo, ZundoOptions, temporal };
