"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  temporal: () => temporal
});
module.exports = __toCommonJS(src_exports);

// src/temporal.ts
var import_zustand = require("zustand");
var createVanillaTemporal = (userSet, userGet, options) => {
  const stateCreator = (set, get) => {
    return {
      pastStates: options?.pastStates || [],
      futureStates: options?.futureStates || [],
      undo: (steps = 1) => {
        if (get().pastStates.length) {
          const pastStates = get().pastStates.slice();
          const futureStates = get().futureStates.slice();
          while (steps--) {
            const pastState = pastStates.pop();
            if (pastState) {
              futureStates.push(options?.partialize?.(userGet()) || userGet());
              userSet(pastState);
            }
          }
          set({ pastStates, futureStates });
        }
      },
      redo: (steps = 1) => {
        if (get().futureStates.length) {
          const pastStates = get().pastStates.slice();
          const futureStates = get().futureStates.slice();
          while (steps--) {
            const futureState = futureStates.pop();
            if (futureState) {
              pastStates.push(options?.partialize?.(userGet()) || userGet());
              userSet(futureState);
            }
          }
          set({ pastStates, futureStates });
        }
      },
      clear: () => set({ pastStates: [], futureStates: [] }),
      isTracking: true,
      pause: () => set({ isTracking: false }),
      resume: () => set({ isTracking: true }),
      setOnSave: (_onSave) => set({ _onSave }),
      // Internal properties
      _onSave: options?.onSave,
      _handleSet: (pastState) => {
        if (get().isTracking) {
          const currentState = options?.partialize?.(userGet()) || userGet();
          if (!options?.equality?.(pastState, currentState)) {
            const pastStates = get().pastStates.slice();
            if (options?.limit && pastStates.length >= options?.limit) {
              pastStates.shift();
            }
            pastStates.push(pastState);
            get()._onSave?.(pastState, currentState);
            set({ pastStates, futureStates: [] });
          }
        }
      }
    };
  };
  return (0, import_zustand.createStore)(
    options?.wrapTemporal?.(stateCreator) || // Cast to a version of the store that does not include "temporal" addition
    stateCreator
  );
};

// src/index.ts
var temporal = (config, options) => {
  const configWithTemporal = (set, get, store) => {
    store.temporal = createVanillaTemporal(set, get, options);
    const curriedHandleSet = options?.handleSet?.(
      store.temporal.getState()._handleSet
    ) || store.temporal.getState()._handleSet;
    const setState = store.setState;
    store.setState = (...args) => {
      const pastState = options?.partialize?.(get()) || get();
      setState(...args);
      curriedHandleSet(pastState);
    };
    return config(
      // Modify the set function to call the userlandSet function
      (...args) => {
        const pastState = options?.partialize?.(get()) || get();
        set(...args);
        curriedHandleSet(pastState);
      },
      get,
      store
    );
  };
  return configWithTemporal;
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  temporal
});
//# sourceMappingURL=index.js.map